-- https://gist.github.com/1383576
-- http://codeplea.com/simple-interpolation
-- http://codeplea.com/introduction-to-splines
 
steprate = 25
 
function smooth( points, open )
        if #points < 3 then
                return points
        end
        
        local steps = steps or 10
        
        local first, second, penult, last = points[1], points[2], points[#points-1], points[#points]
        
        if (not open) then
                table.insert(points,1,last)
                table.insert(points,1,penult)
                points[#points+1] = first
                points[#points+1] = second
        end
        
        local spline = {}
        local start, count = 1, #points - 1
        local p0, p1, p2, p3, x, y
        
        if (not open) then
                start = 2
                count = #points - 2
        end
        
        for i = start, count do
                if open and i == 1 then
                        p0, p1, p2, p3 = points[i], points[i], points[i + 1], points[i + 2]
                        steps = lengthOf( {x=p1.x,y=p1.y} , {x=p3.x,y=p3.y} )
                elseif open and i == count then
                        p0, p1, p2, p3 = points[#points - 2], points[#points - 1], points[#points], points[#points]
                        steps = lengthOf( {x=p1.x,y=p1.y} , {x=p3.x,y=p3.y} )
                else
                        p0, p1, p2, p3 = points[i - 1], points[i], points[i + 1], points[i + 2]
                        steps = lengthOf( {x=p0.x,y=p0.y} , {x=p1.x,y=p1.y} )
                end
                
                for t = 0, 1, 1 / (steps/steprate) do
                        x = 0.5 * ( ( 2 * p1.x ) + ( p2.x - p0.x ) * t + ( 2 * p0.x - 5 * p1.x + 4 * p2.x - p3.x ) * t * t + ( 3 * p1.x - p0.x - 3 * p2.x + p3.x ) * t * t * t )
                        y = 0.5 * ( ( 2 * p1.y ) + ( p2.y - p0.y ) * t + ( 2 * p0.y - 5 * p1.y + 4 * p2.y - p3.y ) * t * t + ( 3 * p1.y - p0.y - 3 * p2.y + p3.y ) * t * t * t )
 
                        --prevent duplicate entries
                        if not(#spline > 0 and spline[#spline].x == x and spline[#spline].y == y) then
                                table.insert( spline , { x = x , y = y } )                              
                        end
                end
        end
        
        if (not open) then
                table.remove(points,1)
                table.remove(points,1)
                table.remove(points,#points)
                table.remove(points,#points)
        end
        
        return spline
end
 
function draw( parent, points, col, radius, aspoints, tapcallback, touchcallback )
        local start, line = 1, nil
        
        if (not aspoints and #points > 1) then
                start = 3
                line = display.newLine( parent, points[1].x, points[1].y, points[2].x, points[2].y )
                line.width = radius
                setColor(line, col)
        end
        
        if (aspoints or (not aspoints and #points > 2)) then
                for i = start, #points do
                        local point = points[i]
                        local c = nil
                        
                        if (aspoints) then
                                c = display.newCircle( parent, point.x, point.y, radius )
                                setColor(c, col)
                        else
                                line:append( point.x, point.y )
                        end
                        
                        if (tapcallback) then c:addEventListener( "tap", tapcallback ) end
                        if (touchcallback) then c:addEventListener( "touch", touchcallback ) end
                end
        end
end
 
--local points = { {x = 100, y = 100}, {x = 100, y = 200}, {x = 200, y = 200}, {x = 200, y = 300} }
 
--local spline = smooth( points )
--draw(spline)